var TMQZ = TMQZ || {};

// Define the Quiz constructor
TMQZ.Quiz = function(quizContainer) {
	this.quizContainer = quizContainer;
	$quizContainer = $(quizContainer);

	// Questions & Answers
	this.questions = quizContainer.getElementsByClassName('tmqz-question');
	this.currentQuestionIndex = 0;
	this.currentQuestion = $(this.questions[this.currentQuestionIndex]);
	this.choices = new Array();

	// Results
	this.results = quizContainer.getElementsByClassName('tmqz-result');

	// Progress
	this.progress = quizContainer.getElementsByClassName('tmqz-progress');

	// Data Attribute Variables
	this.title = $quizContainer.data('quiz-title');
	this.siteTitle = $quizContainer.data('quiz-site-title');
	this.topCategory = $quizContainer.data('quiz-top-category');
	this.shareMessage = $quizContainer.data('quiz-share-template');
	this.selectColor = $quizContainer.data('quiz-select-color');
	this.correctColor = $quizContainer.data('quiz-correct-color');
	this.incorrectColor = $quizContainer.data('quiz-incorrect-color');
	this.correctImage = $quizContainer.data('quiz-correct-image');
	this.incorrectImage = $quizContainer.data('quiz-incorrect-image');

	// Initialize Quiz
	$(this.progress).show();

	// activate first question
	this.activateQuestion(this.currentQuestion);
	this.resizeAnswers();

	// FB init
	this.theActiveClick = '';
	if( typeof window.fbAsyncInit !== 'function' ) {
		window.fbAsyncInit = function() {
			var appIdfromDB = jQuery('meta[property="fb:app_id"]').attr("content");
			if( appIdfromDB === null || typeof appIdfromDB === 'undefined' ) {
				appIdfromDB = '434027466688626';
			}
			FB.init({
				appId : appIdfromDB,
				xfbml : true,
				version : 'v2.6'
			});
			if( this.theActiveClick != '' ) {
				jQuery( this.theActiveClick ).trigger( 'click' );
			}
		}
	}
};

/**
 * TMQZ.Quiz::resetQuiz()
 * Clear score and reset the quiz
 * @param quizContainer
 *
 * NOTE: This only works fully for those properly extended from the base theme
 * TODO: REVEAL & EITHEROR will need to be refactored to take advantage of this functionality
 * (they both currently overwrite this function with their own full locally-scoped logic)
 */
TMQZ.Quiz.prototype.resetQuiz = function() {
	this.currentQuestion.hide();
	$(this.results).hide();

	// clear last round's answers (if present)
	var $answers = $(this.quizContainer).find( '.tmqz-answer' );
	if( $answers.length > 0 ) {
		$answers.removeClass('tmqz-answer-incorrect tmqz-answer-correct tmqz-answer-selected');
		$answers.css('background-color', '');
	}

	// clear last round's text inputs (if present)
	var $text_inputs = $(this.quizContainer).find( 'input[type="text"]' );
	if( $text_inputs.length > 0 ) {
     $text_inputs.val('');
  }

  // clear score (if present)
  if(this.score && this.score > -1) {
		this.score = 0;
		$(this.quizContainer).find( '.tmqz-total-score' ).text( this.score );
	}

	// clear array of selected choices
	if(this.choices) {
		this.choices = [];
	}

  // reset to the first question
	this.currentQuestionIndex = 0;
	this.currentQuestion = $(this.questions[this.currentQuestionIndex]);

	// and show the progress back at number 1
	this.updateProgress();
	$(this.progress).show();
}


TMQZ.Quiz.prototype.assignClicks = function(question) {
	var quizObj = this;

	// Use custom select color
	if (quizObj.selectColor !== '') {
		question.find('.tmqz-answer').hover(function(){
			if ($('.tmqz-answer-correct').length == 0 ||
				$('.tmqz-answer-correct').length == 0 ||
				$('.tmqz-answer-incorrect').length == 0) {

				$(this).css('background-color',quizObj.selectColor);
			}
		}, function() {
			$(this).css('background-color','');
			$('.tmqz-answer-selected').css('background-color',quizObj.selectColor);
			if (quizObj.correctColor !== '') {
				$('.tmqz-answer-correct').css('background-color',quizObj.correctColor);
			}
			if (quizObj.incorrectColor !== '') {
				$('.tmqz-answer-incorrect').css('background-color',quizObj.incorrectColor);
			}
		});
	}

	// Create click events
	question.find('.tmqz-answer').click(function(){
		question.find('.tmqz-answer').unbind('click');

		var answerValue = $(this).data('ans-value');
		quizObj.choices.push(answerValue);

		// Page Tracking
		var qTitle = quizObj.title.replace(/\ /g, '-');
		var questionNum = quizObj.currentQuestionIndex + 1;
		var curQuestion = question.find('.tmqz-question-text h3').text().trim().replace(/\ /g, '-');
		var answer = $(this).text().trim().replace(/\ /g, '-');

		qTitle = qTitle.replace(/[^A-z0-9-_]/g, '');
		curQuestion = curQuestion.replace(/[^A-z0-9-_]/g, '');
		answer = answer.replace(/[^A-z0-9-_]/g, '');

		if (window.cto) {
			cto.trackEvent( { 'engagementType':'Quiz_'+ qTitle + '_' + questionNum + '-' + curQuestion + '_' + answer + '_select-answer|' } );
		}

		$(this).css('background-color','');
		switch(answerValue) {
			case 1:
				$(this).addClass('tmqz-answer-correct');

				// Use custom correct color
				if (quizObj.correctColor !== '') {
					$(this).css('background-color',quizObj.correctColor);
				}

				// Use custom correct image
				if (quizObj.correctImage !== '') {
					$(this).find('.tmqz-answer-result').addClass('tmqz-result-circle')
							.css('background-image',  'url('+quizObj.correctImage+')');
				}

				break;
			case 0:
				$(this).addClass('tmqz-answer-incorrect');
				//question.find('.tmqz-answer[data-ans-value="1"]').addClass('tmqz-answer-correct');

				// Use custom incorrect color
				if (quizObj.incorrectColor !== '') {
					$(this).css('background-color',quizObj.incorrectColor);
				}

				// Use custom incorrect image
				if (quizObj.incorrectImage !== '') {
					$(this).find('.tmqz-answer-result').addClass('tmqz-result-circle')
							.css('background-image',  'url('+quizObj.incorrectImage+')');
				}

				break;
			default:
				$(this).addClass('tmqz-answer-selected');

				// Use custom select color
				if (quizObj.selectColor !== '') {
					$(this).css('background-color',quizObj.selectColor);
				}
		}

		setTimeout(function(){quizObj.nextQuestion();}, 300);
	});
};

TMQZ.Quiz.prototype.unbindClicks = function( currentQuestion ) {
	currentQuestion.find('.tmqz-answer').off('click');
}

TMQZ.Quiz.prototype.nextQuestion = function() {
	this.deactivateQuestion(this.currentQuestion);
	this.currentQuestionIndex++;

	if (this.currentQuestionIndex >= this.questions.length) {
		$(this.progress).hide();
		$(this.getResult()).show();
		$(".tmqz-results").show();
		this.scrollUp();
	} else {
		this.currentQuestion = $(this.questions[this.currentQuestionIndex]);
		this.updateProgress();
		this.activateQuestion(this.currentQuestion);
		this.resizeAnswers();
		this.scrollUp();
	}
};

TMQZ.Quiz.prototype.activateQuestion = function( currentQuestion ) {
	this.assignClicks(currentQuestion);
	currentQuestion.show();
};

TMQZ.Quiz.prototype.deactivateQuestion = function( currentQuestion ) {
	currentQuestion.hide();
	this.unbindClicks(currentQuestion);
};

TMQZ.Quiz.prototype.updateProgress = function() {
	var quizObj = this;
	var currentQuestionCount = quizObj.currentQuestionIndex+1;
	var totalQuestions = quizObj.questions.length;

	// Update Progress Bar
	var progressBarFill = $(quizObj.progress).find('.tmqz-progress-bar-fill');
	var percentDone = (currentQuestionCount/totalQuestions)*100;
	percentDone = percentDone.toString() + "%";
	progressBarFill.width(percentDone);

	// Update Progress Count
	var progressCountCurrent = $(quizObj.progress).find('.tmqz-progress-count-current');
	progressCountCurrent.html( currentQuestionCount );
};

TMQZ.Quiz.prototype.getResult = function() {
	// Placeholder function - Should be defined in child class
	return false;
};

// should be called inside of getResult to allow retry to reset the quiz instead of causing a full page refresh
TMQZ.Quiz.prototype.bindRetry = function( result ) {
	var self = this;

	$(result).find('.tmqz-retry a').on('click', function(e){
		e.preventDefault();
		self.resetQuiz();
		self.activateQuestion( self.currentQuestion );
		$(this).off('click');
	});
};

TMQZ.Quiz.prototype.facebookClick = function( shareUrl, preparedShareMessage, shareDescription, fbImage ) {

	FB.ui({
		method: 'feed',
		link: shareUrl,
		name: preparedShareMessage,
		description: shareDescription,
		picture: fbImage
	}, function(response){});

}

TMQZ.Quiz.prototype.assignSharingClicks = function(result) {
	var quizObj = this;
	var preparedShareMessage = quizObj.replaceTokens(quizObj.shareMessage);
	var shareDescription = $('meta[property="og:description"]').attr('content');
	// TECHNO-494: Space Mountain frameable views should share a canonical link
	// if no link is present, fall back to the window.location.href
	var shareAddress = $('link[rel="canonical"]').attr('href') || window.location.href;

	// Facebook Sharing
	$(result).find('.tmqz-share-fb').click( function(){
		var shareUrl = shareAddress;
		var fbImage = $(result).find('.tmqz-result-image').data('share-image');
		fbImage = fbImage + '?width=1200&height=630&mode=crop';

		if( typeof window.FB != 'undefined' ) {
			quizObj.facebookClick( shareUrl, preparedShareMessage, shareDescription, fbImage );
		} else {
			quizObj.theActiveClick = this;
			/* -- */
			(function(d, s, id){
				var js, fjs = d.getElementsByTagName(s)[0];
				if (d.getElementById(id)) {return;}
				js = d.createElement(s); js.id = id;
				js.src = "//connect.facebook.net/en_US/sdk.js";
				fjs.parentNode.insertBefore(js, fjs);
			}(document, 'script', 'facebook-jssdk'));
			/* -- */
		}

		if (window.cto) {
			cto.trackLink( {'linkName': 'social/quiz-share/social_facebook', 'linkPosition': 'social/quiz' } );
		}

	});

	// Pinterest Sharing
	$(result).find('.tmqz-share-pn').click( function(){
		var shareUrl = 'http://www.pinterest.com/pin/create/button/?url=';
		shareUrl += encodeURIComponent( shareAddress );
		shareUrl += '&media=' + encodeURIComponent( $(result).find('.tmqz-result-image').data('share-image') );
		shareUrl += '&description=' + encodeURIComponent( preparedShareMessage );

		if (window.cto) {
			cto.trackLink( {'linkName': 'social/quiz-share/social_pinterest', 'linkPosition': 'social/quiz' } );
		}

		window.open( shareUrl, "popupWindow", "width=600, height=400, scrollbars=yes" );
	});

	// Twitter Sharing
	$(result).find('.tmqz-share-tw').click( function(){
		var shareUrl = 'https://twitter.com/share?url=';
		shareUrl += encodeURIComponent( shareAddress );
		shareUrl += '&text=' + encodeURIComponent( preparedShareMessage );

		if (window.cto) {
			cto.trackLink( {'linkName': 'social/quiz-share/social_twitter', 'linkPosition': 'social/quiz' } );
		}

		window.open( shareUrl, "popupWindow", "width=600, height=400, scrollbars=yes" );
	});

	// Tumbler Sharing
	$(result).find('.tmqz-share-tb').click( function(){
		var shareUrl = 'https://www.tumblr.com/share/link?url=';
		shareUrl += encodeURIComponent( shareAddress );
		shareUrl += '&name=' + encodeURIComponent( preparedShareMessage );
		shareUrl += '&description=' + encodeURIComponent( shareDescription );

		if (window.cto) {
			cto.trackLink( {'linkName': 'social/quiz-share/social_tumblr', 'linkPosition': 'social/quiz' } );
		}

		window.open( shareUrl, "popupWindow", "width=600, height=400, scrollbars=yes" );
	});

};

TMQZ.Quiz.prototype.replaceTokens = function(string) {
	var quizObj = this;

	var tokenPairs = {
		'total_points': quizObj.questions.length,
		'quiz_title': 	quizObj.title,
		'top_category': quizObj.topCategory,
		'site_title': 	quizObj.siteTitle,
		'score': 				quizObj.score
	};

	$.each(tokenPairs, function(key, val){
		if( val !== undefined ) {
			string = string.replace( '{%' + key + '%}', val );
		}
	});

	return string;
};

TMQZ.Quiz.prototype.resizeAnswers = function() {
	var quizObj = this;

	// Resize text on text only answers
	var answers = quizObj.currentQuestion.find('.tmqz-answer-text');
	answers.css("font-size","");
	answers.each(function() {
		var fSize = 9;
		while (( $( this ).parent().height() < ( $( this ).height() + 24 ) ||
				 $( this ).width() < ( $( this )[0].scrollWidth ) ) && fSize > 5 ) {
			answers.css("font-size","." + fSize + "em");
			fSize--;
		}
	});

	// Resize answers with images AND text
	if (quizObj.currentQuestion.find('.tmqz-answer-image + .tmqz-answer-text').length) {
		answers = quizObj.currentQuestion.find('.tmqz-answer');

		ansHeight = 0;
		answers.css("height","");
		answers.each(function() {
			if (ansHeight < $(this).height()) {
				ansHeight = $(this).height();
			}
		});
		answers.css("height",ansHeight + "px");
	}
};

TMQZ.Quiz.prototype.resize = function() {
	this.resizeAnswers();
};

TMQZ.Quiz.prototype.scrollUp = function() {
	var position = $(this.quizContainer).offset().top - 12;
	if ($(document).scrollTop() > position) {
	   window.scrollTo(0,position); // first value for left offset, second value for top offset
	}
};

// TODO: eliminate these once we have Modernizr
TMQZ.Quiz.prototype.testTouch = function() {
    return !!(('ontouchstart' in window) || window.DocumentTouch && document instanceof DocumentTouch);
};

TMQZ.Quiz.prototype.testiOS = function() {
  return !!window.navigator.userAgent.match(/(ipod|iphone|ipad)/i);
};
